@@ -422,7 +422,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
              'label': _("MIDI:"),
              'players': [
                          ('gst-launch', 'playbin uri=file://%s',
-                                        'filesrc location=exercises/lesson-files/share/fanfare.midi ! wildmidi ! alsasink'),
+                                        'filesrc location=exercises/standard/lesson-files/share/fanfare.midi ! wildmidi ! alsasink'),
                          ('timidity', '-idqq %s'),
                          ('drvmidi', ''),
                          ('playmidi', ''),
@@ -456,7 +456,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             cmd_combo = gtk.ComboBoxEntry(cmd_liststore)
             cmd_combo.set_tooltip_text(_("Enter the name of the program. An absolute path is required only if the executable is not found on the PATH."))
             cmd_combo.opts = gtk.combo_box_entry_new_text()
-            cmd_combo.opts.set_tooltip_text(_("The command line options required. Write %s where you want the name of the file to be played. Or omit it to have it added to the end of the string."))
+            cmd_combo.opts.set_tooltip_text(_("The command line options required. Write %%s where you want the name of the file to be played. Or omit it to have it added to the end of the string.") % ())
             cmd_combo.child.set_text(cfg.get_string('sound/%s_player' % formatid))
             cmd_combo.opts.child.set_text(cfg.get_string('sound/%s_player_options' % formatid))
             for s in format['players']:
@@ -494,7 +494,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             category_vbox.pack_start(box)
     def test_XXX_player(self, w, typeid, testfile):
         try:
-            soundcard.play_mediafile(typeid, os.path.join('exercises/lesson-files/share', testfile))
+            soundcard.play_mediafile(typeid, os.path.join('exercises/standard/lesson-files/share', testfile))
         except osutils.BinaryBaseException, e:
             solfege.win.display_error_message2(e.msg1, e.msg2)
     def create_linux_sound_page(self):
