@@ -1,3 +1,4 @@
+# vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
 # Copyright (C) 2009  Tom Cato Amundsen
 #
@@ -37,128 +38,77 @@ import solfege
 from solfege import cfg
 from solfege import filesystem
 from solfege import gu
-from solfege import learningtree as pd
+from solfege import frontpage as pd
+from solfege import lessonfile
 from solfege import lessonfilegui
 
-class LessonFileDialogCommon(gtk.Dialog):
+class LessonFilePreviewWidget(gtk.VBox):
     def __init__(self, model):
-        gtk.Dialog.__init__(self, _("Select lesson file"), self, buttons=
-            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,))
+        gtk.VBox.__init__(self)
         self.m_model = model
-        lessonfilegui.handle_lesson_id_crash(solfege.app.lessonfile_manager)
-        self.g_ok_button = self.add_button("gtk-ok", gtk.RESPONSE_OK)
-        self.g_ok_button.set_sensitive(False)
-        self.set_default_size(800, 500)
-        vbox = gtk.VBox()
-        self.vbox.pack_start(vbox, False)
-        self.g_liststore = gtk.ListStore(gobject.TYPE_STRING,
-            gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT,
-            gobject.TYPE_STRING)
-        self.update_filelist()
-        self.g_treeview = gtk.TreeView(self.g_liststore)
-        self.g_treeview.connect('cursor-changed', self.on_cursor_changed)
-        renderer = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_("Used"), renderer, text=3)
-        column.set_sort_column_id(3)
-        column.set_resizable(True)
-        self.g_treeview.append_column(column)
-        column = gtk.TreeViewColumn(_("Title"), renderer, text=0)
-        column.set_resizable(True)
-        column.set_sort_column_id(0)
-        self.g_treeview.append_column(column)
-        column = gtk.TreeViewColumn(_("Filename"), renderer, text=1)
-        column.set_resizable(True)
-        self.g_treeview.append_column(column)
-        def sort_func(store, itera, iterb):
-            return cmp(store.get(itera, 1)[0], store.get(iterb, 1)[0])
-        self.g_treeview.get_model().set_sort_func(1, sort_func)
-        column.set_sort_column_id(1)
-        scrolled_window = gtk.ScrolledWindow()
-        scrolled_window.add(self.g_treeview)
-        self.vbox.pack_start(scrolled_window)
-        hbox = gtk.HBox()
-        hbox.set_spacing(8)
-        hbox.set_border_width(12)
-        self.vbox.pack_start(hbox, False)
-        hbox.pack_start(gtk.Label(_("Module:")), False)
+        self.set_size_request(200, 200)
+        l = gtk.Label()
+        l.set_alignment(0.0, 0.5)
+        l.set_markup("<b>Title:</b>")
+        self.pack_start(l, False)
+        self.g_title = gtk.Label()
+        self.g_title.set_alignment(0.0, 0.5)
+        self.pack_start(self.g_title, False)
+        l = gtk.Label()
+        l.set_alignment(0.0, 0.5)
+        l.set_markup("<b>Module:</b>")
+        self.pack_start(l, False)
         self.g_module = gtk.Label()
-        hbox.pack_start(self.g_module, False)
-        #
-        hbox = gtk.HBox()
-        hbox.set_spacing(8)
-        hbox.set_border_width(12)
-        self.vbox.pack_start(hbox, False)
-        hbox.pack_start(gtk.Label(_("Used by topic(s):")), False)
-        self.g_topics_vbox = gtk.VBox()
-        hbox.pack_start(self.g_topics_vbox, False)
-        #
+        self.g_module.set_alignment(0.0, 0.5)
+        self.pack_start(self.g_module, False)
+        l = gtk.Label()
+        l.set_alignment(0.0, 0.5)
+        l.set_markup("<b>Used in topcis:</b>")
+        self.pack_start(l, False)
+        self.g_topic_box = gtk.VBox()
+        self.pack_start(self.g_topic_box, False)
         self.show_all()
-        self.g_treeview.connect('row-activated', self.on_row_activated)
-    def get_selected_lesson_id(self):
-        """
-        Returns None if no lesson file is selected.
-        """
-        i = self.g_treeview.get_cursor()[0]
-        if i:
-            i = i[0]
-            iter = self.g_liststore.iter_nth_child(None, i)
-            return self.g_liststore.get(iter, 2)[0]
-    def on_row_activated(self, treeview, row, treeviewcolumn):
-        self.response(gtk.RESPONSE_OK)
-    def on_cursor_changed(self, treeview):
-        i = self.g_treeview.get_cursor()[0][0]
-        iter = self.g_liststore.iter_nth_child(None, i)
-        self.g_module.set_text(self.g_liststore.get(iter, 4)[0])
-        self.g_topics_vbox.foreach(lambda w: w.destroy())
-        for s in self.m_model.iterate_topics_for_id(
-                self.g_liststore.get(iter, 2)[0]):
-            label = gtk.Label(s)
-            label.set_alignment(0.0, 0.5)
-            label.show()
-            self.g_topics_vbox.pack_start(label)
-        self.g_ok_button.set_sensitive(True)
+    def update(self, dlg):
+        fn = dlg.get_preview_filename()
+        if fn:
+            fn = gu.decode_filename(fn)
+            for child in self.g_topic_box.get_children():
+                child.destroy()
+            fn = lessonfile.mk_uri(fn)
+            try:
+                self.set_sensitive(True)
+                self.g_title.set_text(lessonfile.infocache.get(fn, 'title'))
+                self.g_module.set_text(lessonfile.infocache.get(fn, 'module'))
+                self.g_ok_button.set_sensitive(True)
+                for x in self.m_model.iterate_topics_for_file(fn):
+                    l = gtk.Label(x)
+                    l.set_alignment(0.0, 0.5)
+                    self.g_topic_box.pack_start(l, False)
+                if not self.g_topic_box.get_children():
+                    l = gtk.Label(u"-")
+                    l.set_alignment(0.0, 0.5)
+                    self.g_topic_box.pack_start(l, False)
+            except (lessonfile.InfoCache.FileNotFound, 
+                    lessonfile.InfoCache.FileNotLessonfile), e:
+                self.g_title.set_text(u'')
+                self.g_module.set_text(u'')
+                self.g_ok_button.set_sensitive(False)
+                self.set_sensitive(False)
+        self.show_all()
+        return True
 
-
-class SelectLessonFileDialog(LessonFileDialogCommon):
-    def __init__(self, model):
-        LessonFileDialogCommon.__init__(self, model)
-        hbox = gtk.HBox()
-        hbox.set_spacing(gu.hig.SPACE_SMALL)
-        hbox.show()
-        self.vbox.pack_start(hbox, False)
-        self.vbox.reorder_child(hbox, 0)
-        g_show_used = gu.nCheckButton('SelectLessonFileDialog',
-            'show_used_files', _("_Show files used in other topics"),
-            False, self.update_filelist)
-        hbox.pack_start(g_show_used, False)
-        b = gtk.Button(_("_Search for new lesson files"))
-        b.connect('clicked', self.search_for_lesson_files)
-        b.show()
-        hbox.pack_start(b, False)
-        self.g_progress = gtk.ProgressBar()
-        self.g_progress.set_text(_("Parsing files"))
-        hbox.pack_start(self.g_progress, False)
-    def search_for_lesson_files(self, btn):
-        self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
-        self.g_progress.show()
-        for percent in solfege.app.nonidle_reparse():
-            self.g_progress.set_fraction(percent if percent <= 1.0 else 1.0)
-            while gtk.events_pending():
-                gtk.main_iteration()
-        self.g_progress.hide()
-        self.update_filelist()
-        self.window.set_cursor(None)
-    def update_filelist(self, w=None):
-        self.g_liststore.clear()
-        use_count = self.m_model.get_use_dict()
-        if cfg.get_bool('SelectLessonFileDialog/show_used_files'):
-            lesson_id_list = list(solfege.app.lessonfile_manager.iterate_lesson_ids())
-        else:
-            lesson_id_list = [i for i in list(solfege.app.lessonfile_manager.iterate_lesson_ids()) if use_count.get(i, 0) == 0]
-        for lesson_id in lesson_id_list:
-            d = solfege.app.lessonfile_manager.m_uiddb[lesson_id]
-            self.g_liststore.append((d['header']['title'], d['filename'], lesson_id, use_count.get(lesson_id, 0),
-                solfege.app.lessonfile_manager.get(lesson_id, 'module')))
+class SelectLessonFileDialog(gtk.FileChooserDialog):
+    def __init__(self, parent):
+        gtk.FileChooserDialog.__init__(self, _("Select lesson file"),
+            parent=parent,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,))
+        self.set_select_multiple(True)
+        pv = LessonFilePreviewWidget(parent.m_model)
+        pv.g_ok_button = self.add_button("gtk-ok", gtk.RESPONSE_OK)
+        pv.g_ok_button.set_sensitive(False)
+        pv.show()
+        self.set_preview_widget(pv)
+        self.connect('selection-changed', pv.update)
 
 
 def editor_of(obj):
@@ -315,15 +265,34 @@ class Section(gtk.VBox):
         menu.show_all()
         menu.popup(None, None, None, event.button, event.time)
     def on_add_link(self, btn):
-        dlg = SelectLessonFileDialog(editor_of(self).m_model)
-        ret = dlg.run()
-        if ret in (gtk.RESPONSE_REJECT, gtk.RESPONSE_DELETE_EVENT, gtk.RESPONSE_CANCEL):
-            pass
+        if editor_of(self).m_filename:
+            open_dir = os.path.split(editor_of(self).m_filename)[0]
         else:
-            assert ret == gtk.RESPONSE_OK
-            self.m_model.append(dlg.get_selected_lesson_id())
-            self.g_link_box.pack_start(
-                self.create_linkrow(dlg.get_selected_lesson_id()), False)
+            open_dir = filesystem.user_data()
+        dlg = SelectLessonFileDialog(editor_of(self))
+        dlg.set_current_folder(open_dir)
+        while 1:
+            ret = dlg.run()
+            if ret in (gtk.RESPONSE_REJECT, gtk.RESPONSE_DELETE_EVENT, gtk.RESPONSE_CANCEL):
+                break
+            else:
+                assert ret == gtk.RESPONSE_OK
+                for filename in dlg.get_filenames():
+                    fn = gu.decode_filename(filename)
+                    assert os.path.isabs(fn)
+                    # If the file name is a file in a subdirectory below
+                    # lessonfile.exercises_dir in the current working directory,
+                    # then the file is a standard lesson file, and it will be
+                    # converted to a uri scheme with:
+                    fn = lessonfile.mk_uri(fn)
+                    # Small test to check that the file actually is a lesson file.
+                    try:
+                        lessonfile.infocache.get(fn, 'title')
+                    except lessonfile.infocache.FileNotLessonfile:
+                        continue
+                    self.m_model.append(fn)
+                    self.g_link_box.pack_start(self.create_linkrow(fn), False)
+                break
         dlg.destroy()
     def on_add_link_to_new_page(self, menuitem):
         page = pd.Page(_("Untitled%s") % "", [pd.Column()])
@@ -344,7 +313,13 @@ class Section(gtk.VBox):
             linkbutton = gu.ClickableLabel(link_this.m_name)
             linkbutton.connect('clicked', ff, link_this)
         else:
-            linkbutton = gu.ClickableLabel(solfege.db.get_field(link_this, 'title'))
+            try:
+                linkbutton = gu.ClickableLabel(lessonfile.infocache.get(link_this, 'title'))
+                linkbutton.set_tooltip_text(link_this)
+            except lessonfile.InfoCache.FileNotFound:
+                linkbutton = gu.ClickableLabel(_(u"«%s» was not found") % link_this)
+                linkbutton.make_warning()
+
         hbox.pack_start(linkbutton)
         linkbutton.connect('button-press-event', self.on_right_click_row, link_this)
         hbox.show_all()
@@ -586,9 +561,42 @@ class Clipboard(list):
         for button, types in self.m_paste_buttons:
             button.set_sensitive(bool(self) and isinstance(self[-1], types))
 
+class PropertiesDialog(gtk.Dialog):
+    def __init__(self, model, parent):
+        gtk.Dialog.__init__(self, "Properties", parent,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
+                     gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        vbox = gu.hig_dlg_vbox()
+        self.vbox.pack_start(vbox)
+        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        #
+        label = gtk.Label()
+        label.set_alignment(1.0, 0.5)
+        sizegroup.add_widget(label)
+        label.set_markup(u"<b>%s</b>" % _("Filename:"))
+        label2 = gtk.Label(parent.m_filename)
+        label2.set_alignment(0.0, 0.5)
+        hbox = gtk.HBox()
+        hbox.set_spacing(gu.hig.SPACE_MEDIUM)
+        hbox.pack_start(label, False)
+        hbox.pack_start(label2, True, True)
+        vbox.pack_start(hbox, False)
+        #
+        label = gtk.Label()
+        label.set_alignment(1.0, 0.5)
+        label.set_markup(u"<b>%s</b>" % _("Front page title:"))
+        sizegroup.add_widget(label)
+        self.g_title_entry = gtk.Entry()
+        self.g_title_entry.set_text(model.m_name)
+        hbox = gtk.HBox()
+        hbox.set_spacing(gu.hig.SPACE_MEDIUM)
+        hbox.pack_start(label, False)
+        hbox.pack_start(self.g_title_entry, True, True)
+        vbox.pack_start(hbox, False)
+
 
 class Editor(gtk.Window, gu.EditorDialogBase):
-    savedir = os.path.join(filesystem.user_data(), 'learningtrees')
+    savedir = os.path.join(filesystem.user_data(), u'exercises')
     # The clipboard will be shared between all Editor instances
     clipboard = Clipboard()
     def __init__(self, filename=None):
@@ -600,6 +608,7 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         self.add(self.g_main_box)
         self.g_actiongroup.add_actions([
             ('GoBack', gtk.STOCK_GO_BACK, None, None, None, self.go_back),
+            ('Properties', gtk.STOCK_PROPERTIES, None, None, None, self.properties),
         ])
         self.setup_toolbar()
         # This dict maps the windows created for all pages belonging to
@@ -617,6 +626,7 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         self.g_main_box.pack_start(self.m_page_mapping[id(self.m_model)])
         self.clipboard.update_buttons()
         self.show_all()
+        self.add_to_instance_dict()
     def __del__(self):
         logging.debug("fpeditor.Editor.__del__, filename=" % self.m_filename)
     def add_page(self, page):
@@ -639,6 +649,13 @@ class Editor(gtk.Window, gu.EditorDialogBase):
             not isinstance(self.g_visible_page.m_parent, Editor))
     def go_back(self, *action):
         self.show_page(self.g_visible_page.m_parent)
+    def properties(self, *action):
+        dlg = PropertiesDialog(self.m_model, self)
+        dlg.show_all()
+        ret = dlg.run()
+        if ret == gtk.RESPONSE_ACCEPT:
+            self.m_model.m_name = dlg.g_title_entry.get_text()
+        dlg.destroy()
     def setup_toolbar(self):
         self.g_ui_manager.insert_action_group(self.g_actiongroup, 0)
         uixml = """
@@ -649,6 +666,7 @@ class Editor(gtk.Window, gu.EditorDialogBase):
           <toolitem action='Open'/>
           <toolitem action='Save'/>
           <toolitem action='SaveAs'/>
+          <toolitem action='Properties'/>
           <toolitem action='Close'/>
           <toolitem action='Help'/>
          </toolbar>
@@ -681,17 +699,10 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         else:
             try:
                 win = Editor(fn)
+                win.show()
             except IOError, e:
                 gu.dialog_ok(_("Loading file '%(filename)s' failed: %(msg)s") %
                         {'filename': fn, 'msg': str(e).decode('utf8', 'replace')})
-            win.show()
-    @staticmethod
-    def tuple_to_filename(place, filename):
-        if place == 'solfege':
-            return os.path.join("learningtrees", filename)
-        else:
-            assert place == 'user'
-            return os.path.join(filesystem.user_data(), 'learningtrees', filename)
     def load_file(self, filename):
         """
         Load a file into a empty, newly created Editor object.
@@ -730,13 +741,13 @@ class Editor(gtk.Window, gu.EditorDialogBase):
         return self.is_modified()
     def save(self, w=None):
         assert self.m_filename
-        f = open(self.m_filename, 'w')
-        self.m_model.dump(f)
-        f.close()
+        save_location = os.path.split(self.m_filename)[0] + os.sep
+        fh = pd.FileHeader(1, self.m_model)
+        fh.save_file(self.m_filename)
         self.set_not_modified()
         # We do test for solfege.win since it is not available during testing
         if getattr(solfege, 'win', None):
-            solfege.win.on_learning_tree_changed()
+            solfege.win.on_frontpage_changed()
     def on_show_help(self, *w):
         return
 
