@@ -607,7 +607,7 @@ class Voice:
         key = (music.m_pitch.steps(), music.m_pitch.m_accidental_i)
         if __debug__:
             if key in self.m_coldict[timepos].m_music:
-                logging.warning("Voice: warning, adding the same notehead twice %s" % key)
+                logging.warning("Voice: warning, adding the same notehead twice %s" % str(key))
             if (self.m_coldict[timepos].m_duration is not None) \
                     and (self.m_coldict[timepos].m_duration != music.m_duration):
                 logging.warning("mpd: warning: All noteheads on the same stem should have the same length")
